'''
 Scripts used to make figures in GenGIS: A geospatial information system for genomic data (Genome Res., 2009).
'''

import GenGisAPIv1 as ggAPI
import dataHelper as dh
import movieHelper as mh
import math

gLineIds = []

def playMovie():
	pos1 = [ggAPI.Point3D(0,0, 2.0940396785736084), 36, 20]
	pos2 = [ggAPI.Point3D(0,0, 2.0940396785736084), 306, 20]
	pos3 = [ggAPI.Point3D(-0.21385592222213745,0.23764827847480774, 0.31232169270515442), 249, 20]
	
	mh.linearInterpolateParams(pos1, pos1, 1)
	mh.linearInterpolateParams(pos1, pos2, 5)
	mh.linearInterpolateParams(pos2, pos3, 5)


def projectToOceanicSites():
	oceanicSites = ['GS002', 'GS003', 'GS004', 'GS007', 'GS008', 'GS009', 'GS010', 'GS013', 'GS014', 'GS015', 'GS016', 'GS017', 'GS018', 'GS019']
	
	geoTreeCtrl = ggAPI.study.GetMap(0).GetTree(0).GetGeoTreeController()
	geoTreeCtrl.ProjectTree(dh.createVec(oceanicSites))
	ggAPI.refresh()
	
	
def restoreTree():
	geoTreeCtrl = ggAPI.study.GetMap(0).GetTree(0).GetGeoTreeController()
	geoTreeCtrl.RestoreTree()
	ggAPI.refresh()
	
	
def setChartLocations():
	positions = [ggAPI.Point3D(0.207017108798,-4.21109834861e-05,-0.637511134148)]
	positions.append(ggAPI.Point3D(0.348798751831,0.00113510980736,-0.945418179035))
	positions.append(ggAPI.Point3D(0.780016064644,-4.21091244789e-05,-0.790091753006))
	positions.append(ggAPI.Point3D(0.670043885708,0.00118413835298,-0.939836382866))
	positions.append(ggAPI.Point3D(0.498665243387,-4.21079166699e-05,-0.889327406883))
	positions.append(ggAPI.Point3D(0.196332708001,0.00167432660237,-0.882434666157))
	positions.append(ggAPI.Point3D(-0.0395620241761,0.00103704631329,-0.866062998772))
	positions.append(ggAPI.Point3D(0.010181046091,0.00069376785541,-0.549492835999))
	positions.append(ggAPI.Point3D(-0.247250318527,0.00123315956444,-0.480730950832))
	positions.append(ggAPI.Point3D(-0.199632301927,0.00126971618738,-0.688452661037))
	positions.append(ggAPI.Point3D(-0.393661171198,0.00147826527245,-0.315927714109))
	positions.append(ggAPI.Point3D(-0.17965221405,0.00010509169806,-0.215360492468))
	positions.append(ggAPI.Point3D(-0.466057300568,0.000350401212927,-0.0942597836256))
	positions.append(ggAPI.Point3D(-0.373283058405,-4.21203003498e-05,0.128200516105))
	positions.append(ggAPI.Point3D(-0.596933960915,-4.21208278567e-05,0.171700552106))
	positions.append(ggAPI.Point3D(-0.654871404171,-4.2123872845e-05,0.421922832727))
	positions.append(ggAPI.Point3D(-0.47944021225,-4.2125757318e-05,0.576845645905))
	positions.append(ggAPI.Point3D(-0.343133747578,-5.06489559484e-05,0.745212197304))
	positions.append(ggAPI.Point3D(-0.476348280907,-4.213055945e-05,0.971385240555))

	locSet = dh.getLocationSetData()
	
	for i in xrange(0,len(positions)):
		chart = locSet[i].GetChart()
		chart.SetCentre(positions[i])
		
	ggAPI.refresh()
	
	
def selectTaxa(taxa):
	locData = dh.getLocationSetData()
	for loc in locData:
		seqData = loc.sequences
		for seq in seqData:
			if seq.data['Class'] in taxa:
				seq.SetActive(True)
			else:
				seq.SetActive(False)
			
	dh.getChartSetView().UpdateCharts("Class")
	ggAPI.refresh()
	
	
def drawUniqueSeqCount():
	ggAPI.clearLines()
	
	# get all selected locations
	locData = dh.getLocationSetData()
	visibleLocSites = []
	for loc in locData:
		if loc.GetVisibility():
			visibleLocSites.append(loc)
	
	# get normalized number of unique sequences
	normUniqueSeqs = []
	for loc in visibleLocSites:
		uniqueSeq = set([])
		    
		seqs = loc.sequences
		for seq in seqs:
			uniqueSeq.add(seq.sequenceId)
    	
		normUniqueSeqs.append(float(len(uniqueSeq))/len(seqs))
		
	# normalize values between [0.0,1.0]
	maxValue = max(normUniqueSeqs)
	minValue = min(normUniqueSeqs)
	for i in xrange(0, len(normUniqueSeqs)):
		normUniqueSeqs[i] = (normUniqueSeqs[i] - minValue) / (maxValue - minValue)

	# draw lines for each site
	for i in xrange(0, len(visibleLocSites)):
		pos = ggAPI.Point3D()
		ggAPI.convertGeoCoord(visibleLocSites[i].northing, visibleLocSites[i].easting, pos)
				
		endPos = ggAPI.Point3D(pos.x, normUniqueSeqs[i]*0.5 + 0.05, pos.z)
		line = ggAPI.VisualLine(ggAPI.Colour(1,1-normUniqueSeqs[i],1-normUniqueSeqs[i]), 6, ggAPI.LINE_STYLE.SOLID, ggAPI.Line3D(pos, endPos))
		lineId = ggAPI.addLine(line)
		
	ggAPI.refresh()

